{{/* vim: set filetype=mustache: */}}

{{/* Namespace */}}
{{- define "app.namespace" -}}
{{- if gt (len (.Release.Namespace)) 63 -}}
{{- fail (printf ".Release.Namespace %s ist zu lang (max. 63 Zeichen)" .Release.Namespace) -}}
{{- end -}}
{{ printf "%s" .Release.Namespace }}
{{- end -}}

{{/* Chart: Name + Version */}}
{{- define "app.chart" -}}
{{- if gt (len (printf "%s-%s" .Chart.Name .Chart.Version)) 63 -}}
{{- fail (printf ".Chart.Name-.Chart.Version %s-%s ist zu lang (max. 63 Zeichen)" .Chart.Name .Chart.Version) -}}
{{- end -}}
{{ printf "%s-%s" .Chart.Name .Chart.Version }}
{{- end -}}

{{/* Managed-by -> On Helm, this value is always Helm */}}
{{- define "app.managedBy" -}}
{{- if gt (len (.Release.Service)) 63 -}}
{{- fail (printf ".Release.Service %s ist zu lang (max. 63 Zeichen)" .Release.Service) -}}
{{- end -}}
{{ printf "%s" .Release.Service }}
{{- end -}}

{{/* Default Labels: Helm recommended best-practice labels https://helm.sh/docs/chart_best_practices/labels/ */}}
{{- define "app.defaultLabels" }}
app.kubernetes.io/instance: vorgang-manager
app.kubernetes.io/managed-by: {{ include "app.managedBy" . }}
app.kubernetes.io/name: {{ .Release.Name }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
app.kubernetes.io/part-of: ozgcloud
app.kubernetes.io/version: {{ .Chart.Version }}
helm.sh/chart: {{ include "app.chart" . }}
ozgcloud-mongodb-client: "true"
{{- end -}}

{{- define "app.matchLabels" }}
app.kubernetes.io/name: {{ .Release.Name }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
{{- end -}}

{{- define "app.envSpringProfiles" }}
{{- if (.Values.env).overrideSpringProfiles -}}
{{ printf "%s" (.Values.env).overrideSpringProfiles }}
{{- else -}}
{{ printf "oc, %s" (include "app.ozgcloudEnvironment" . ) }}
{{- end -}}
{{- end -}}

{{- define "app.ozgcloudEnvironment" -}}
{{- required "ozgcloud.environment must be set" (.Values.ozgcloud).environment -}}
{{- end -}}

{{- define "app.vorgangManagerDatabase" -}}
{{- if (.Values.database).databaseName -}}
{{  .Values.database.databaseName }}
{{- else -}}
{{ printf "vorgang-manager-database" }}
{{- end -}}
{{- end -}}

{{- define "app.grpc_client_user-manager_address" -}}
{{ printf "dns:///%s.%s:9000" .Values.usermanagerName .Release.Namespace }}
{{- end -}}

{{- define "app.ozgcloud_migration_user-manager_address" -}}
{{ printf "http://%s.%s:8080/migration/user" .Values.usermanagerName .Release.Namespace }}
{{- end -}}

{{- define "app.ozgcloud_vorgangmanager_address" -}}
{{ printf "dns:///%s.%s:9090" .Values.vorgangmanagerName .Release.Namespace }}
{{- end -}}

{{- define "app.ozgcloud_administration_address" -}}
{{ printf "http://%s.%s:8080" .Values.administrationName .Release.Namespace }}
{{- end -}}

{{- define "app.getApplicationAddress" -}}
{{- if .Values.applicationAddress -}}
{{  .Values.applicationAddress }}
{{- else -}}
{{ printf "dns:///%s.%s:9090" .Values.vorgangmanagerName .Release.Namespace }}
{{- end -}}
{{- end -}}

{{- define "app.getNachrichtenManagerAddress" -}}
{{- if .Values.nachrichtenManagerAddress -}}
{{  .Values.nachrichtenManagerAddress }}
{{- else -}}
{{ printf "%s.%s:9090" .Values.vorgangmanagerName .Release.Namespace }}
{{- end -}}
{{- end -}}

{{- define "app.databaseSecretName" -}}
{{- if (.Values.database).secretName -}}
{{  .Values.database.secretName }}
{{- else -}}
{{  printf "ozg-mongodb-admin-vorgang-manager-user" }}
{{- end -}}
{{- end -}}

{{- define "app.serviceAccountName" -}}
{{ printf "%s" ( (.Values.serviceAccount).name | default "vorgang-manager-service-account" ) }}
{{- end -}}

{{- define "app.getCustomList" -}}
{{- with (.Values.env).customList -}}
{{- if kindIs "map" . -}}
{{ include "app.dictToList" . }}
{{- else if kindIs "slice" . -}}
{{ . | toYaml }}
{{- end -}}
{{- end -}}
{{- end -}}

{{- define "app.dictToList" -}}
{{- $customList := list -}}
{{- range $key, $value := . -}}
{{- $customList = append $customList (dict "name" $key "value" $value) }}
{{- end -}}
{{- $customList | toYaml -}}
{{- end -}}


{{- define "app.bayernidAbsenderName" -}}
{{- quote (required "ozgcloud.bayernid.absender.name must be set if ozgcloud.bayernid is enabled" (((.Values.ozgcloud).bayernid).absender).name) -}}
{{- end -}}

{{- define "app.bayernidAbsenderMandant" -}}
{{- if (((.Values.ozgcloud).bayernid).absender).mandant -}}
{{ quote .Values.ozgcloud.bayernid.absender.mandant }}
{{- else -}}
{{ include "app.bayernidAbsenderName" . }}
{{- end -}}
{{- end -}}

{{- define "app.bayernidAbsenderDienst" -}}
{{- if (((.Values.ozgcloud).bayernid).absender).dienst -}}
{{  quote .Values.ozgcloud.bayernid.absender.dienst }}
{{- else -}}
{{ include "app.bayernidAbsenderName" . }}
{{- end -}}
{{- end -}}

{{- define "app.elsterTransferOperatorNamespace" -}}
{{- required "elsterTransferOperator.namespace must be set" (.Values.elsterTransferOperator).namespace -}}
{{- end -}}

{{- define "app.ozgcloudBezeichner" -}}
{{- required "ozgcloud.bezeichner must be set" (.Values.ozgcloud).bezeichner -}}
{{- end -}}

{{- define "app.baseUrl" -}}
{{- required "baseUrl must be set" .Values.baseUrl }}
{{- end -}}

{{- define "app.alfaHost" -}}
{{- if .Values.alfaHost -}}
{{ .Values.alfaHost | quote }}
{{- else -}}
{{ printf "https://%s.%s" (include "app.ozgcloudBezeichner" .) (include "app.baseUrl" .) }}
{{- end -}}
{{- end -}}

{{- define "app.fachstelleHost" -}}
{{- if (.Values.fachstelle).enabled -}}
{{- required "fachstelle.host must be set" .Values.fachstelle.host | quote }}
{{- end -}}
{{- end -}}

{{- define "app.getZufiManagerAddress" -}}
{{- required "zufiManager.address must be set" (.Values.zufiManager).address -}}
{{- end -}}

{{- define "app.getKeyStorePath" -}}
{{- printf "/workspace/keystore/xta-keystore.p12" -}}
{{- end -}}

{{- define "app.getTrustStorePath" -}}
{{- printf "/workspace/truststore/xta-truststore.jks" -}}
{{- end -}}