/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.callcontext;

import static de.ozgcloud.common.grpc.GrpcUtil.*;

import java.util.UUID;

import io.grpc.CallOptions;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.ClientInterceptor;
import io.grpc.ForwardingClientCall.SimpleForwardingClientCall;
import io.grpc.Metadata;
import io.grpc.MethodDescriptor;

public class VorgangManagerClientCallContextAttachingInterceptor implements ClientInterceptor {

	static final String KEY_REQUEST_ID = "REQUEST_ID-bin";
	static final String KEY_CLIENT_NAME = "CLIENT_NAME-bin";

	public static final String VORGANG_MANAGER_CLIENT_NAME = "OzgCloud_VorgangManager";

	// <A> = Request, <B> = Response
	@Override
	public <A, B> ClientCall<A, B> interceptCall(MethodDescriptor<A, B> method, CallOptions callOptions, Channel next) {
		return new CallContextAttachingClientCall<>(next.newCall(method, callOptions));
	}

	final class CallContextAttachingClientCall<A, B> extends SimpleForwardingClientCall<A, B> {

		protected CallContextAttachingClientCall(ClientCall<A, B> delegate) {
			super(delegate);
		}

		@Override
		public void start(Listener<B> responseListener, Metadata headers) {
			headers.merge(buildCallContextMetadata());
			super.start(responseListener, headers);
		}

		private Metadata buildCallContextMetadata() {
			var metadata = new Metadata();

			metadata.put(createKeyOf(KEY_REQUEST_ID), UUID.randomUUID().toString().getBytes());
			metadata.put(createKeyOf(KEY_CLIENT_NAME), VORGANG_MANAGER_CLIENT_NAME.getBytes());

			return metadata;
		}
	}
}
