/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.vorgang.callcontext.CurrentUserService;
import de.ozgcloud.vorgang.common.security.PolicyService;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceImplBase;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAcknowledgeResponse;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcDeleteClientAttributeRequest;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcUpdateClientAttributeRequest;
import io.grpc.stub.StreamObserver;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
public class GrpcClientAttributeService extends ClientAttributeServiceImplBase {

	@Autowired
	private ClientAttributeMapper mapper;
	@Autowired
	private ClientAttributeService service;
	@Autowired
	private PolicyService policyService;
	@Autowired
	private CurrentUserService userService;

	@Override
	public void set(GrpcSetClientAttributeRequest request, StreamObserver<GrpcAcknowledgeResponse> responseObserver) {
		policyService.checkPermission(Set.of(request.getVorgangId()));

		service.set(request.getVorgangId(), mapForSet(request.getAttribute()));

		responseObserver.onNext(GrpcAcknowledgeResponse.newBuilder().build());
		responseObserver.onCompleted();
	}

	private ClientAttribute mapForSet(GrpcClientAttribute clientAttribute) {
		return mapper.mapFromGrpc(clientAttribute).toBuilder().clientName(userService.getUser().getClientName()).build();
	}

	@Override
	public void update(GrpcUpdateClientAttributeRequest request, StreamObserver<GrpcAcknowledgeResponse> responseObserver) {
		policyService.checkPermission(Set.of(request.getVorgangId()));

		service.update(request.getVorgangId(), mapper.mapFromGrpc(request.getAttribute()));

		responseObserver.onNext(GrpcAcknowledgeResponse.newBuilder().build());
		responseObserver.onCompleted();
	}

	@Override
	public void delete(GrpcDeleteClientAttributeRequest request, StreamObserver<GrpcAcknowledgeResponse> responseObserver) {
		policyService.checkPermission(Set.of(request.getVorgangId()));

		service.delete(buildIdentificator(request));

		responseObserver.onNext(GrpcAcknowledgeResponse.newBuilder().build());
		responseObserver.onCompleted();
	}

	private ClientAttributeIdentificator buildIdentificator(GrpcDeleteClientAttributeRequest request) {
		return ClientAttributeIdentificator.builder()
				.vorgangId(request.getVorgangId())
				.clientName(request.getClientName())
				.attributeName(request.getAttributeName())
				.build();
	}

}
