/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import de.ozgcloud.command.Command;

import lombok.extern.log4j.Log4j2;
import net.javacrumbs.shedlock.spring.annotation.SchedulerLock;

@Component
@Profile("!itcase")
@Log4j2
public class CommandDeletionScheduler {

	static final int CLEANUP_THRESHOLD_MINUTES = 30;

	@Autowired
	private CommandService commandService;

	@Scheduled(fixedDelayString = "${ozgcloud.vorgang-manager.command.scheduler.fixedDelay:3600000}", //
			initialDelayString = "${ozgcloud.vorgang-manager.command.scheduler.initialDelay:1800000}")
	@SchedulerLock(name = "CommandDeletionScheduler")
	void deleteCommands() {
		try (var deleteVorgangCommands = commandService.findFinishedVorgangLoeschenCommandsOlderThen(getCleanupThreshold())) {
			deleteVorgangCommands.map(Command::getVorgangId).distinct()
					.forEach(commandService::deleteAllByVorgang);
		} catch (RuntimeException e) {
			LOG.error("Error deleting commands", e);
		}
	}

	ZonedDateTime getCleanupThreshold() {
		return ZonedDateTime.now(ZoneOffset.UTC).minusMinutes(CLEANUP_THRESHOLD_MINUTES);
	}

}
