/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.vorgang.grpc.command.GrpcAddSubCommandsRequest;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Mapper(uses = { GrpcCreateCommandMapper.class }, nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE,
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, unmappedTargetPolicy = ReportingPolicy.IGNORE)
abstract class GrpcAddSubCommandsRequestMapper {

	@Mapping(target = "executionMode", expression = "java(mapExecutionMode(request.getExecutionMode()))")
	@Mapping(target = "subCommands", source = "subCommandsList")
	abstract CreateSubCommandsRequest from(GrpcAddSubCommandsRequest request);

	SubCommandExecutionMode mapExecutionMode(String executionModeValue) {
		if (notParallelExecutionMode(executionModeValue)) {
			LOG.error("Unsupported execution mode: {}. {} will be used instead", executionModeValue, SubCommandExecutionMode.PARALLEL);
		}
		return SubCommandExecutionMode.PARALLEL;
	}

	boolean notParallelExecutionMode(String executionMode) {
		return !SubCommandExecutionMode.PARALLEL.name().equalsIgnoreCase(executionMode);
	}

}
