/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import java.time.ZonedDateTime;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.vorgang.callcontext.CallContext;
import de.ozgcloud.vorgang.callcontext.CurrentUserService;
import de.ozgcloud.vorgang.callcontext.User;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.files.FileIdMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcUser;
import de.ozgcloud.vorgang.vorgang.redirect.RedirectRequestMapper;

@Mapper(uses = { RedirectRequestMapper.class, CommandBodyMapper.class, FileIdMapper.class, GrpcObjectMapper.class }, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
public abstract class GrpcCreateCommandRequestMapper {

	@Autowired
	private CurrentUserService userService;

	@Mapping(target = "bodyObject", source = "bodyObj")
	@Mapping(target = "body", source = "bodyList")
	@Mapping(target = "order", source = "orderString")
	@Mapping(target = "callContext", expression = "java(fromHeaderOrCall(grpcRequest.getCallContext()))")
	abstract CreateCommandRequest fromGrpc(GrpcCreateCommandRequest grpcRequest);

	CallContext fromHeaderOrCall(GrpcCallContext callContext) {
		return userService.findUser().map(user -> CallContext.builder()
				.client(user.getClientName())
				.user(User.builder().id(user.getUserId().orElse(null)).name(user.getUserName().orElse(null)).build()).build())
				.orElseGet(() -> fromGrpc(callContext));

	}

	abstract CallContext fromGrpc(GrpcCallContext context);

	abstract User fromGrpc(GrpcUser user);

	ZonedDateTime map(String localDate) {
		if (StringUtils.isBlank(localDate)) {
			return null;
		}

		return ZonedDateTime.parse(localDate);
	}

}
