/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.grpc;

import java.util.function.Supplier;

import io.grpc.stub.StreamObserver;
import lombok.Setter;

public class GrpcResponseBatchStreamer<E, T> {

	public static final int DEFAULT_BATCH_SIZE = 100;

	@Setter
	private int batchSize = DEFAULT_BATCH_SIZE;

	private final Supplier<GrpcResponseBuilder<E, T>> responseBuilderSupplier;
	private final StreamObserver<T> responseObserver;
	private GrpcResponseBuilder<E, T> responseBuilder;

	public static <E, T> GrpcResponseBatchStreamer<E, T> create(Supplier<GrpcResponseBuilder<E, T>> responseBuilderSupplier,
			StreamObserver<T> responseObserver) {
		return new GrpcResponseBatchStreamer<>(responseBuilderSupplier, responseObserver);
	}

	private GrpcResponseBatchStreamer(Supplier<GrpcResponseBuilder<E, T>> responseBuilderSupplier,
			StreamObserver<T> responseObserver) {
		this.responseBuilderSupplier = responseBuilderSupplier;
		this.responseObserver = responseObserver;
		responseBuilder = responseBuilderSupplier.get();
	}

	public void send(E element) {
		responseBuilder.addElement(element);
		if (batchIsFull()) {
			sendResponse();
			responseBuilder = responseBuilderSupplier.get();
		}
	}

	private boolean batchIsFull() {
		return responseBuilder.getElementCount() >= batchSize;
	}

	public void finish() {
		if (!batchIsEmpty()) {
			sendResponse();
		}
		responseObserver.onCompleted();
	}

	private boolean batchIsEmpty() {
		return responseBuilder.getElementCount() == 0;
	}

	private void sendResponse() {
		responseObserver.onNext(responseBuilder.build());
	}
}
