/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.index.CompoundIndexDefinition;
import org.springframework.data.mongodb.core.index.IndexDefinition;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2024-06-12 17:00:00 OZG-5661", order = "M010", author = "ebardin")
public class M010_CreateIndexesInVorgang { // NOSONAR

	static final String COLLECTION_NAME = "vorgang";
	static final String COMPOUND_INDEX_NAME = "inCreation_OrgaId_assignedTo_status_createdAt";

	static final String FIELD_IN_CREATION = "inCreation";
	static final String FIELD_STATUS = "status";
	static final String FIELD_CREATED_AT = "createdAt";
	static final String FIELD_ASSIGNED_TO = "assignedTo";
	static final String FIELD_EINGANGS = "eingangs";
	static final String FIELD_ORGANISATIONSEINHEITEN_ID = FIELD_EINGANGS + ".zustaendigeStelle.organisationseinheitenId";

	@Execution
	public void doMigration(MongoTemplate template) {
		template.indexOps(COLLECTION_NAME).ensureIndex(buildCompoundIndexDefinition());
	}

	IndexDefinition buildCompoundIndexDefinition() {
		var indexKeys = new Document();
		indexKeys.put(FIELD_IN_CREATION, 1);
		indexKeys.put(FIELD_ORGANISATIONSEINHEITEN_ID, 1);
		indexKeys.put(FIELD_ASSIGNED_TO, 1);
		indexKeys.put(FIELD_STATUS, 1);
		indexKeys.put(FIELD_CREATED_AT, 1);
		return new CompoundIndexDefinition(indexKeys).named(COMPOUND_INDEX_NAME);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}

}
