package de.ozgcloud.vorgang.common.migration;

import org.bson.Document;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.MergeOperation;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2024-11-27 09:00:00 OZG-6810", order = "M013", author = "ebardin", runAlways = true)
public class M013_CreateOzgCloudFileCollection { // NOSONAR

	static final String OLD_COLLECTION_NAME = "fs.files";
	static final String NEW_COLLECTION_NAME = "ozgCloudFile";
	static final String FIELD_ID = "_id";
	static final String FIELD_VERSION = "version";
	static final String FIELD_FILE_NAME = "name";
	static final String FIELD_CONTENT_TYPE = "contentType";
	static final String FIELD_CLIENT = "client";
	static final String FIELD_VORGANG_ID = "vorgangId";
	static final String FIELD_CREATED_BY = "createdBy";
	static final String FIELD_FIELD_NAME = "fieldName";
	static final String FIELD_CONTENT_ID = "contentId";
	static final String FIELD_SIZE = "size";
	static final String FIELD_LENGTH = "length";
	static final String SET_VERSION_EXPRESSION = "{$literal: 0}";
	static final String SET_CONTENT_ID_EXPRESSION = "{$toString: \"$%s\" }".formatted(FIELD_ID);

	@Execution
	public void doMigration(MongoTemplate template) {
		template.aggregate(buildAggregationPipeline(), OLD_COLLECTION_NAME, Document.class);
	}

	private Aggregation buildAggregationPipeline() {
		return Aggregation.newAggregation(
				Aggregation.project(FIELD_ID)
						.andExpression(SET_VERSION_EXPRESSION).as(FIELD_VERSION)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_VORGANG_ID).as(FIELD_VORGANG_ID)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_CLIENT).as(FIELD_CLIENT)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_FIELD_NAME).as(FIELD_FIELD_NAME)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_CONTENT_TYPE).as(FIELD_CONTENT_TYPE)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_FILE_NAME).as(FIELD_FILE_NAME)
						.and(RenameUtil.METADATA_KEY + "." + FIELD_CREATED_BY).as(FIELD_CREATED_BY)
						.andExpression(SET_CONTENT_ID_EXPRESSION).as(FIELD_CONTENT_ID)
						.and(FIELD_LENGTH).as(FIELD_SIZE),
				Aggregation.merge().intoCollection(NEW_COLLECTION_NAME).on(FIELD_ID)
						.whenDocumentsMatch(MergeOperation.WhenDocumentsMatch.keepExistingDocument()).build()
		);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}