/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import java.time.ZonedDateTime;

import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.DateFormat;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;
import org.springframework.data.elasticsearch.annotations.Setting;

import lombok.Builder;
import lombok.Getter;
import lombok.ToString;

@Document(indexName = "#{@environment.getProperty('ozgcloud.elasticsearch.index')}")
@Setting(settingPath = "elastic/settings.json")
@ToString
@Getter
@Builder
class IndexedVorgang {

	static final String FIELD_ID = "vorgangId";
	static final String FIELD_NAME = "vorgangName";
	static final String FIELD_VORGANG_NUMMER = "vorgangNummer";
	static final String FIELD_ANTRAGSTELLER_NAME = "antragstellerName";
	static final String FIELD_ANTRAGSTELLTER_VORNAME = "antragstellerVorname";
	static final String FIELD_AKTENZEICHEN = "aktenzeichen";
	static final String FIELD_ORGANISATIONSEINHEITEN_ID = "organisationseinheitenId";
	static final String FIELD_CREATED_AT = "createdAt";
	static final String FIELD_STATUS = "status";
	static final String FIELD_ASSIGNED_TO = "assignedTo";

	@Id
	private String vorgangId;

	@Field(type = FieldType.Keyword, normalizer = "lowercase_normalizer")
	private String vorgangName;
	@Field(type = FieldType.Keyword, normalizer = "lowercase_normalizer")
	private String vorgangNummer;

	@Field(type = FieldType.Date, format = DateFormat.date_time, pattern = "yyyy-MM-dd'T'HH:mm:ss'Z'")
	private ZonedDateTime createdAt;
	@Field(type = FieldType.Keyword, normalizer = "lowercase_normalizer")
	private String antragstellerName;
	@Field(type = FieldType.Keyword, normalizer = "lowercase_normalizer")
	private String antragstellerVorname;

	@Field(type = FieldType.Keyword, normalizer = "lowercase_normalizer")
	private String aktenzeichen;
	private String status;
	private String organisationseinheitenId;
	private String assignedTo;
}
