/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.ozgcloud.vorgang.vorgang.Antragsteller;
import de.ozgcloud.vorgang.vorgang.Eingang;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangHeader;
import de.ozgcloud.vorgang.vorgang.ZustaendigeStelle;

@Mapper(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
interface IndexedVorgangMapper {

	@Mapping(target = "vorgangName", source = "name")
	@Mapping(target = "vorgangNummer", source = "nummer")
	@Mapping(target = "antragstellerName", expression = "java(mapAntragstellerName(vorgang))")
	@Mapping(target = "antragstellerVorname", expression = "java(mapAntragstellerVorname(vorgang))")
	@Mapping(target = "organisationseinheitenId", expression = "java(mapOrganistationseinheitenId(vorgang))")
	@Mapping(target = "vorgangId", source = "id")
	IndexedVorgang fromVorgang(Vorgang vorgang);

	default String mapAntragstellerName(Vorgang vorgang) {
		var builder = new StringBuilder();
		getAntragsteller(vorgang).ifPresent(antragsteller -> builder.append(antragsteller.getNachname()));
		return builder.toString();
	}

	default String mapAntragstellerVorname(Vorgang vorgang) {
		var builder = new StringBuilder();
		getAntragsteller(vorgang).ifPresent(antragsteller -> builder.append(antragsteller.getVorname()));
		return builder.toString();
	}

	private Optional<Antragsteller> getAntragsteller(Vorgang vorgang) {
		return Optional.ofNullable(vorgang.getEingangs().get(0).getAntragsteller());
	}

	default String mapOrganistationseinheitenId(Vorgang vorgang) {
		var builder = new StringBuilder();
		getZustaendigeStelle(vorgang.getEingangs().get(0))
				.ifPresent(zustaendigeStelle -> builder.append(zustaendigeStelle.getOrganisationseinheitenId()));
		return builder.toString();
	}

	private Optional<ZustaendigeStelle> getZustaendigeStelle(Eingang eingang) {
		return Optional.ofNullable(eingang.getZustaendigeStelle());
	}

	@Mapping(target = "clientAttributes", ignore = true)
	@Mapping(target = "formEngineName", ignore = true)
	@Mapping(target = "id", source = "vorgangId")
	@Mapping(target = "name", source = "vorgangName")
	@Mapping(target = "nummer", source = "vorgangNummer")
	@Mapping(target = "version", ignore = true)
	VorgangHeader toVorgangHeader(IndexedVorgang vorgang);
}
