/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import java.util.Arrays;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.TextCriteria;
import org.springframework.data.mongodb.core.query.TextQuery;
import org.springframework.data.support.PageableExecutionUtils;
import org.springframework.stereotype.Repository;

import de.ozgcloud.vorgang.vorgang.FindVorgangRequest;
import de.ozgcloud.vorgang.vorgang.VorgangHeader;
import lombok.RequiredArgsConstructor;

@ConditionalOnProperty(prefix = "ozgcloud.mongodbsearch", name = "enabled", havingValue = "true")
@RequiredArgsConstructor
@Repository
public class SearchVorgangCustomRepositoryMongo implements SearchVorgangCustomRepository {

	private final MongoOperations mongoOperations;

	@Override
	public Page<VorgangHeader> searchBy(final FindVorgangRequest request) {
		var query = buildQuery(request);
		var vorgangs = mongoOperations.stream(query, VorgangHeader.class).skip(request.getOffset()).limit(request.getLimit()).toList();
		return PageableExecutionUtils.getPage(vorgangs, buildPageable(request), vorgangs::size);
	}

	private Query buildQuery(FindVorgangRequest request) {
		return TextQuery.queryText(buildTextCriteria(request.getSearchBy())).sortByScore();
	}

	TextCriteria buildTextCriteria(String searchBy) {
		var textCriteria = TextCriteria.forDefaultLanguage();
		Arrays.stream(searchBy.split(StringUtils.SPACE)).forEach(textCriteria::matchingPhrase);
		return textCriteria;
	}

	PageRequest buildPageable(FindVorgangRequest request) {
		return PageRequest.of(request.getOffset() / request.getLimit(), request.getLimit());
	}

	@Override
	public void update(final String vorangId, final Map<String, Object> patch) {
		// update implemented in vorgang.vorgang.VorgangRepository
	}
}
