/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.io.InputStream;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Stream;

import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import de.ozgcloud.vorgang.common.security.PolicyService;
import de.ozgcloud.vorgang.vorgang.IncomingFile;
import lombok.RequiredArgsConstructor;

//TODO make service package protected, as soon PersistPostfachMailByCommandService and FileStreamService is gone
@Service
@RequiredArgsConstructor
public class FileService {

	public static final int CHUNK_SIZE = 255 * 1024;

	private final PolicyService policyService;
	private final EingangFilesRepository repository;
	private final OzgCloudFileRepository ozgCloudFileRepository;
	private final FileIdMapper fileIdMapper;

	public List<IncomingFile> getAttachments(String vorgangId, Optional<String> eingangId) {
		policyService.checkPermission(vorgangId);
		return repository.findAttachments(vorgangId, eingangId);
	}

	public List<IncomingFile> getRepresentations(String vorgangId, Optional<String> eingangId) {
		policyService.checkPermission(vorgangId);
		return repository.findRepresentations(vorgangId, eingangId);
	}

	public FileId saveOzgCloudFile(OzgCloudFile ozgCloudFile) {
		return fileIdMapper.toFileId(ozgCloudFileRepository.save(ozgCloudFile).getId());
	}

	public void patch(FileId fileId, long version, Map<String, Object> patch) {
		ozgCloudFileRepository.patch(fileId, version, patch);
	}

	@Async
	public CompletableFuture<String> uploadFileContentStreamAsync(UploadedFilesReference ref, InputStream content) {
		return CompletableFuture.completedFuture(uploadFileContent(ref, content));
	}

	String uploadFileContent(UploadedFilesReference ref, InputStream content) {
		return ozgCloudFileRepository.uploadContent(ref, content);
	}

	public InputStream getUploadedFileStream(FileId fileId) {
		return ozgCloudFileRepository.getFileContent(fileId);
	}

	public Stream<OzgCloudFile> findOzgCloudFiles(Collection<FileId> ids) {
		return ozgCloudFileRepository.findAll(ids);
	}

	public void deleteAllByVorgang(String vorgangId) {
		ozgCloudFileRepository.deleteAllByVorgang(vorgangId);
	}

	public void deleteContent(String contentId) {
		ozgCloudFileRepository.deleteContent(contentId);
	}
}