/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.registry;

import java.util.concurrent.TimeUnit;

import jakarta.annotation.PostConstruct;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import de.ozgcloud.vorgang.common.userconfig.UserConfigService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.javacrumbs.shedlock.spring.annotation.SchedulerLock;

@Component
@ConditionalOnProperty(prefix = "ozgcloud.zufi-manager", name = { "address" })
@Log4j2
@RequiredArgsConstructor
class RegistryScheduler {

	private final RegistryService registryService;

	private final UserConfigService userConfigService;

	@PostConstruct
	void logInitialization() {
		LOG.info("Starting ZuFi registry scheduler");
	}

	@Scheduled(fixedDelayString = "${ozgcloud.registry.scheduler.fixedDelayMinutes}",
			initialDelayString = "${ozgcloud.registry.scheduler.initialDelayMinutes}",
			timeUnit = TimeUnit.MINUTES)
	@SchedulerLock(name = "RegistryScheduler")
	void register() {
		try {
			registryService.registerVorgangManager(userConfigService.getSupportedOrganisationsEinheiten());
		} catch (Exception e) {
			LOG.error("Error registering VorgangManager", e);
		}
	}
}
