/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.servicekonto;

import java.util.List;
import java.util.Set;

import jakarta.validation.constraints.NotEmpty;

import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;
import lombok.extern.log4j.Log4j2;

@Validated
@Getter
@Builder
@Log4j2
public class ServiceKonto {

	public static final String OSI_IDENTIFIER = "OSI";
	private static final Set<String> EXPECTED_TRUST_LEVELS = Set.of("STORK-QAA-Level-1", "STORK-QAA-Level-2", "STORK-QAA-Level-3",
			"STORK-QAA-Level-4");

	private String type;
	private String trustLevel;

	@NotEmpty
	@Singular
	private List<PostfachAddress> postfachAddresses;

	public static class ServiceKontoBuilder {

		public ServiceKontoBuilder trustLevel(String trustLevel) {
			validateTrustLevel(trustLevel);
			this.trustLevel = trustLevel;
			return this;
		}

		void validateTrustLevel(String trustLevel) {
			if (StringUtils.isNotEmpty(trustLevel) && !EXPECTED_TRUST_LEVELS.contains(trustLevel)) {
				logError(trustLevel);
			}
		}

		void logError(String trustLevelValue) {
			LOG.error("Unexpected trust level: {}", trustLevelValue);
		}
	}
}