/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.status;

import static de.ozgcloud.vorgang.common.db.CriteriaUtil.*;
import static de.ozgcloud.vorgang.vorgang.Vorgang.*;
import static org.springframework.data.mongodb.core.query.Query.*;

import java.util.Map;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import com.mongodb.client.result.UpdateResult;

import de.ozgcloud.vorgang.common.db.CollisionVerifier;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import lombok.extern.log4j.Log4j2;

@Repository
@Log4j2
class StatusRepository {

	@Autowired
	private MongoOperations mongoOperations;

	private CollisionVerifier collisionVerifier;

	@PostConstruct
	void init() {
		collisionVerifier = createCollisionVerifier();
	}

	CollisionVerifier createCollisionVerifier() {
		return new CollisionVerifier(this::doVorgangExist);
	}

	boolean doVorgangExist(String vorgangId) {
		return mongoOperations.exists(query(isId(vorgangId)), Vorgang.COLLECTION_NAME);
	}

	public void updateStatusWithoutVersionCheck(String vorgangId, Status status) {
		var update = new Update()
				.inc(MONGODB_FIELDNAME_VERSION, 1)
				.set(MONGODB_FIELDNAME_STATUS, status);
		updateFirst(new Criteria(MONGODB_FIELDNAME_ID).is(vorgangId), update);
	}

	public void patch(String vorgangId, long version, Map<String, Object> patch) {
		var update = new Update().inc(MONGODB_FIELDNAME_VERSION, 1);
		patch.forEach(update::set);

		collisionVerifier.verify(updateFirst(whereIdAndVersion(vorgangId, version), update), vorgangId);
	}

	private UpdateResult updateFirst(Criteria queryObj, Update update) {
		var result = mongoOperations.updateFirst(query(queryObj), update, Vorgang.class);
		LOG.debug("Update result: {}", result);
		return result;
	}

}
