/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;

import org.mapstruct.Mapper;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.vorgang.servicekonto.ServiceKontoMapper;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN, //
		unmappedSourcePolicy = ReportingPolicy.WARN, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		uses = ServiceKontoMapper.class)
interface EingangHeaderMapper {

	EingangHeader fromGrpc(GrpcEingangHeader grpcEingangHeader);

	public static ZonedDateTime dateStrToZonedDateTime(String dateStr) {
		return ZonedDateTime.parse(dateStr);
	}

	GrpcEingangHeader toGrpc(EingangHeader eingangHeader);

	public static String zonedDateTimeToDateStr(ZonedDateTime zdt) {
		return DateTimeFormatter.ISO_OFFSET_DATE_TIME.format(zdt);
	}
}