/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import org.mapstruct.Mapper;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.vorgang.common.operator.OperandFunctionParser;
import de.ozgcloud.vorgang.common.operator.Operator;
import de.ozgcloud.vorgang.common.operator.OperatorBuilder;

@Mapper
abstract class FindVorgangQueryMapper {

	@Autowired
	private OperandFunctionParser operandFunctionParser;

	public FindVorgangQuery fromQuery(GrpcQuery query) {
		var findVorgangQueryBuilder = FindVorgangQuery.builder()
				.logicalOperator(fromLogicalOperator(query.getLogicalOperator()));
		query.getExpressionsList().stream().map(this::buildOperator).forEach(findVorgangQueryBuilder::operator);
		if (hasSubQuery(query)) {
			findVorgangQueryBuilder.nestedQuery(fromQuery(query.getNestedQuery()));
		}
		return findVorgangQueryBuilder.build();
	}

	boolean hasSubQuery(GrpcQuery query) {
		return !GrpcQuery.getDefaultInstance().equals(query.getNestedQuery());
	}

	FindVorgangQuery.LogicalOperator fromLogicalOperator(GrpcLogicalOperator operator) {
		return FindVorgangQuery.LogicalOperator.valueOf(operator.name());
	}

	Operator buildOperator(GrpcVorgangQueryExpression queryExpression) {
		return OperatorBuilder.from(queryExpression.getOperator())
				.fieldPath(queryExpression.getPath())
				.operand(getValue(queryExpression)).build();
	}

	Object getValue(GrpcVorgangQueryExpression queryExpression) {
		return switch (queryExpression.getOperandCase()) {
			case OPERANDBOOLVALUE -> queryExpression.getOperandBoolValue();
			case OPERANDINTVALUE -> queryExpression.getOperandIntValue();
			case OPERANDSTRINGVALUE -> getStringValue(queryExpression);
			case OPERAND_NOT_SET -> null;
		};
	}

	String getStringValue(GrpcVorgangQueryExpression queryExpression) {
		var operandStringValue = queryExpression.getOperandStringValue();
		if ("null".equalsIgnoreCase(operandStringValue)) {
			return null;
		} else if (operandStringValue.endsWith(")")) {
			return operandFunctionParser.parse(operandStringValue);
		}
		return operandStringValue;
	}
}
