/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import com.google.protobuf.ByteString;

import de.ozgcloud.eingang.forwarding.GrpcAttachmentFile;
import de.ozgcloud.eingang.forwarding.GrpcRepresentationFile;
import de.ozgcloud.vorgang.files.FileIdMapper;

@Mapper(uses = FileIdMapper.class)
public interface IncomingFileMapper {

	IncomingFile mapGrpc(GrpcIncomingFile file);

	default byte[] map(ByteString value) {
		return value.toByteArray();
	}

	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "contentTypeBytes", ignore = true)
	@Mapping(target = "fileNameBytes", ignore = true)
	@Mapping(target = "groupNameBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vendorIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "fileName", source = "file.name")
	GrpcAttachmentFile toAttachmentFile(String groupName, IncomingFile file);

	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "contentTypeBytes", ignore = true)
	@Mapping(target = "fileNameBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vendorIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "fileName", source = "name")
	GrpcRepresentationFile toRepresentationFile(IncomingFile file);
}