/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.command.Command;

@Mapper
public interface LockMapper {

	@Mapping(target = "clientName", source = "createdByClientName")
	@Mapping(target = "lockedSince", expression = "java(getLockedSince())")
	@Mapping(target = "reason", source = "bodyObject")
	Lock fromCommand(Command command);

	default String getReason(Map<String, Object> bodyObject) {
		return StringUtils.trimToNull(MapUtils.getString(bodyObject, Lock.FIELD_REASON));
	}

	default ZonedDateTime getLockedSince() {
		return ZonedDateTime.now(ZoneOffset.UTC);
	}
}
