/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static java.util.Objects.*;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.stereotype.Component;

import de.ozgcloud.vorgang.common.operator.Operator;
import de.ozgcloud.vorgang.vorgang.FindVorgangQuery.LogicalOperator;

@Component
public class QueryCriteriaBuilder {

	public Criteria from(FindVorgangQuery query) {
		var criterias = new ArrayList<Criteria>();
		if (nonNull(query.getNestedQuery())) {
			criterias.add(from(query.getNestedQuery()));
		}
		addFromOperators(criterias, query.getOperators());
		return connect(query.getLogicalOperator(), criterias);
	}

	void addFromOperators(Collection<Criteria> criterias, Collection<Operator> operators) {
		operators.stream().map(Operator::getCriteria).forEach(criterias::add);
	}

	Criteria connect(LogicalOperator logicalOperator, Collection<Criteria> criterias) {
		if (criterias.isEmpty()) {
			return new Criteria();
		}
		if (criterias.size() == 1) {
			return criterias.iterator().next();
		}
		return switch (logicalOperator) {
			case AND -> new Criteria().andOperator(criterias);
			case OR -> new Criteria().orOperator(criterias);
		};
	}
}
