/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static java.util.Objects.*;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.vorgang.common.search.SearchService;

@Service
class VorgangHeaderService {

	@Autowired
	private VorgangHeaderRepository repository;

	@Autowired
	private VorgangService vorgangService;

	@Autowired(required = false)
	private SearchService searchService;

	public Page<VorgangHeader> findAll(FindVorgangRequest request) {
		if (nonNull(searchService) && nonNull(request.getSearchBy())) {
			return removeUnpermitted(searchService.find(request));
		}
		return removeUnpermitted(repository.findAll(request));
	}

	private Page<VorgangHeader> removeUnpermitted(Page<VorgangHeader> loaded) {
		var content = loaded.getContent().stream()
				.map(this::removeUnpermittedClientAttributes)
				.toList();

		return new PageImpl<>(content, loaded.getPageable(), loaded.getTotalElements());
	}

	private VorgangHeader removeUnpermittedClientAttributes(VorgangHeader vorgangHeader) {
		return vorgangHeader.toBuilder()
				.clientAttributes(vorgangService.removeUnpermitted(vorgangHeader.getClientAttributes()))
				.build();
	}

	public Optional<VorgangHeader> findById(String vorgangId) {
		return repository.findById(vorgangId);
	}

	public VorgangHeader getById(String vorgangId) {
		return findById(vorgangId).orElseThrow(() -> new TechnicalException(vorgangId + " could not be found"));
	}
}