/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import java.util.Map;

import org.apache.commons.collections4.MapUtils;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.ozgcloud.command.Command;
import de.ozgcloud.eingang.forwarding.GrpcEingangStub;
import de.ozgcloud.eingang.forwarding.GrpcRouteCriteria;
import de.ozgcloud.eingang.forwarding.GrpcRouteForwarding;
import de.ozgcloud.vorgang.vorgang.Eingang;
import de.ozgcloud.vorgang.vorgang.EingangMapper;
import de.ozgcloud.vorgang.vorgang.GrpcEingang;

@Mapper(imports = MapUtils.class, uses = EingangMapper.class, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
interface ForwardingRequestMapper {

	interface CommandBodyFields {
		String ORGANISATION_EINHEIT_ID = "organisationEinheitId";
		String NAME = "name";
		String STRASSE = "strasse";
		String HAUSNUMMER = "hausnummer";
		String PLZ = "plz";
		String ORT = "ort";
	}

	@Mapping(target = "version", source = "relationVersion")
	@Mapping(target = "organisationEinheit", source = "bodyObject")
	ForwardingRequest fromCommand(Command command);

	@Mapping(target = "id", expression = "java(MapUtils.getString(body, CommandBodyFields.ORGANISATION_EINHEIT_ID))")
	@Mapping(target = "name", expression = "java(MapUtils.getString(body, CommandBodyFields.NAME))")
	@Mapping(target = "address", source = ".")
	OrganisationEinheit toOrganisationEinheit(Map<String, Object> body);

	@Mapping(target = "street", expression = "java(MapUtils.getString(body, CommandBodyFields.STRASSE))")
	@Mapping(target = "houseNumber", expression = "java(MapUtils.getString(body, CommandBodyFields.HAUSNUMMER))")
	@Mapping(target = "zipCode", expression = "java(MapUtils.getString(body, CommandBodyFields.PLZ))")
	@Mapping(target = "city", expression = "java(MapUtils.getString(body, CommandBodyFields.ORT))")
	Address toAddress(Map<String, Object> body);

	@Mapping(target = "routeCriteria", source = "request")
	@Mapping(target = "eingangStub", source = "eingang")
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	GrpcRouteForwarding toGrpcRouteForwarding(ForwardingRequest request, Eingang eingang);

	@Mapping(target = "organisationEinheitId", source = "organisationEinheit.id")
	GrpcRouteCriteria toGrpcRouteCriteria(ForwardingRequest request);

	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	GrpcEingangStub toGrpcEingangStub(GrpcEingang eingang);
}
