/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.osi;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.nachrichten.postfach.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.StringBasedIdentifier;
import de.ozgcloud.vorgang.VorgangManagerServerApplication;

@Disabled("Echter Test für OSI Postfach mit echtem Zugriff. Unbedingt defaultmäßig deaktivieren!!")
@SpringBootTest(
		classes = { VorgangManagerServerApplication.class, OsiPostfachProperties.class },
		properties = {
				"ozgcloud.osi.postfach.proxyapi.url=https://postfach.serviceportal-stage.schleswig-holstein.de/ApiProxy/V1/Message",
				"ozgcloud.osi.postfach.proxyapi.key=db03d369-438a-4c1a-bcb8-9de951f5ef41",
				"ozgcloud.osi.postfach.proxyapi.realm=urn:osp:names:realm:stage:sh",
				"ozgcloud.osi.postfach.scheduler.enabled=false"
		})
@ITCase
class OsiPostfachITCase {

	@Autowired
	private OsiPostfachRemoteService service;

	@Test
	void shouldGetAllMessages() {
		Stream<PostfachNachricht> messages = service.getAllMessages();

		assertThat(messages).hasSizeGreaterThan(0);
	}

	@Test
	@Disabled("Echter Test für OSI Postfach mit echtem Zugriff. Unbedingt defaultmäßig deaktivieren!!")
	void shouldSendMessage() {

		PostfachNachricht nachricht = PostfachNachrichtTestFactory.createBuilder()
				.postfachAddress(PostfachAddressTestFactory.createBuilder()
						.serviceKontoType(OsiPostfachRemoteService.POSTFACH_TYPE)
						.identifier(StringBasedIdentifier.builder().postfachId("2f5c78f8-4891-456f-269b-08d937ab9a72").build())
						.build()
				)
				.vorgangId("98877665544").build();

		assertDoesNotThrow(() -> service.sendMessage(nachricht));
	}
}
