/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.attached_item;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;

class GrpcVorgangAttachedItemMapperTest {

	@Spy
	@InjectMocks
	private GrpcVorgangAttachedItemMapper mapper = Mappers.getMapper(GrpcVorgangAttachedItemMapper.class);
	@Spy
	private GrpcObjectMapper objectMapper = Mappers.getMapper(GrpcObjectMapper.class);

	@Nested
	class TestMapTo {

		@Test
		void shouldContainVorgangAttachedItem() {
			var vorgangAttachedItem = mapTo(VorgangAttachedItemTestFactory.create());

			assertThat(vorgangAttachedItem).isNotNull();
		}

		@Test
		void shouldMapFields() {
			var vorgangAttachedItem = mapTo(VorgangAttachedItemTestFactory.createBuilder().id(VorgangAttachedItemTestFactory.ID).build());

			assertThat(vorgangAttachedItem.getId()).isEqualTo(VorgangAttachedItemTestFactory.ID);
			assertThat(vorgangAttachedItem.getVersion()).isEqualTo(VorgangAttachedItemTestFactory.VERSION);
			assertThat(vorgangAttachedItem.getItemName()).isEqualTo(VorgangAttachedItemTestFactory.ITEM_NAME);
			assertThat(vorgangAttachedItem.getItem()).isNotNull();
			assertThat(vorgangAttachedItem.getVorgangId()).isEqualTo(VorgangTestFactory.ID);
		}

		@Test
		void shouldMapItemFields() {
			var vorgangAttachedItem = mapTo(VorgangAttachedItemTestFactory.createBuilder().id(VorgangAttachedItemTestFactory.ID).build());

			assertThat(vorgangAttachedItem.getItem().getPropertyCount()).isEqualTo(1);
			assertThat(vorgangAttachedItem.getItem().getProperty(0)).isNotNull();
			assertThat(vorgangAttachedItem.getItem().getProperty(0).getName()).isEqualTo(VorgangAttachedItemTestFactory.ITEM_FIELD_NAME);
			assertThat(vorgangAttachedItem.getItem().getProperty(0).getValue(0))
					.isEqualTo(VorgangAttachedItemTestFactory.ITEM_FIELD_STRING_VALUE);
		}

		private GrpcVorgangAttachedItem mapTo(VorgangAttachedItem vorgangAttachedItem) {
			return mapper.mapTo(vorgangAttachedItem);
		}
	}
}