/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import static de.ozgcloud.vorgang.common.migration.M008_UpdateMailServiceClientNameInGridFs.*;
import static org.assertj.core.api.Assertions.*;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Query;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class M008_UpdateMailServiceClientNameInGridFsITCase {

	private static final Document OLD_CLIENT_NAME_GRID_FS_FILE = RenameTestFactory.createGridFsFile(OLD_CLIENT_NAME);

	private final M008_UpdateMailServiceClientNameInGridFs migration = new M008_UpdateMailServiceClientNameInGridFs();

	@Autowired
	private MigrationDbTestUtils dbTestUtils;
	@Autowired
	private MongoOperations db;

	@BeforeEach
	public void init() {
		dbTestUtils.dropGridFsFilesCollection();
	}

	@DisplayName("Do migration")
	@Nested
	class TestDoMigration {

		@BeforeEach
		void init() {
			dbTestUtils.saveGridFsFile(OLD_CLIENT_NAME_GRID_FS_FILE);
		}

		@Test
		void shouldUpdateClientInMetadata() {
			migration.doMigration(db);

			var metadata = (Document) MapUtils.getObject(getGridFsFile(), RenameUtil.METADATA_KEY);
			assertThat(metadata).contains(entry(RenameUtil.CLIENT_KEY, NEW_CLIENT_NAME));
		}

		@Test
		void shouldUpdateClientInFilename() {
			migration.doMigration(db);

			var filename = MapUtils.getString(getGridFsFile(), RenameUtil.FILENAME_KEY);
			var expectedFilename = StringUtils.replace(RenameTestFactory.FILE_NAME_VALUE, OLD_CLIENT_NAME, NEW_CLIENT_NAME);
			assertThat(filename).isEqualTo(expectedFilename);
		}

		@Test
		void shouldKeepBaseData() {
			migration.doMigration(db);

			var gridFsFile = getGridFsFile();
			assertThat(gridFsFile).contains(entry(RenameTestFactory.LENGTH_KEY, RenameTestFactory.LENGTH_VALUE))
					.contains(entry(RenameTestFactory.CHUNK_SIZE_KEY, RenameTestFactory.CHUNK_SIZE_VALUE))
					.contains(entry(RenameTestFactory.UPLOAD_DATE_KEY, RenameTestFactory.UPLOAD_DATE_VALUE));
		}

		@Test
		void shouldKeepMetadata() {
			migration.doMigration(db);

			var metadata = (Document) MapUtils.getObject(getGridFsFile(), RenameUtil.METADATA_KEY);
			assertThat(metadata).contains(entry(RenameTestFactory.VORGANG_ID_KEY, RenameTestFactory.VORGANG_ID_VALUE))
					.contains(entry(RenameTestFactory.FIELD_NAME_KEY, RenameTestFactory.FIELD_NAME_VALUE))
					.contains(entry(RenameTestFactory.CONTENT_TYPE_KEY, RenameTestFactory.CONTENT_TYPE_VALUE))
					.contains(entry(RenameTestFactory.NAME_KEY, RenameTestFactory.NAME_VALUE));
		}

		private Document getGridFsFile() {
			return dbTestUtils.findGridFsFile(new Query()).get(0);
		}
	}
}