/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.migration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.index.IndexDefinition;
import org.springframework.data.mongodb.core.index.IndexOperations;

class M010_CreateIndexesInVorgangTest {

	@Spy
	@InjectMocks
	private M010_CreateIndexesInVorgang migration;

	@Mock
	private MongoTemplate template;
	@Mock
	private IndexOperations indexOperations;

	@Nested
	class TestDoMigration {

		@Mock
		private IndexDefinition indexDefinition;

		@BeforeEach
		void init() {
			when(template.indexOps(anyString())).thenReturn(indexOperations);
		}

		@Test
		void shouldGetIndexOperations() {
			migration.doMigration(template);

			verify(template).indexOps(M010_CreateIndexesInVorgang.COLLECTION_NAME);
		}

		@Test
		void shouldCallBuildCompoundIndexDefinition() {
			migration.doMigration(template);

			verify(migration).buildCompoundIndexDefinition();
		}

		@Test
		void shouldEnsureComponentIndex() {
			doReturn(indexDefinition).when(migration).buildCompoundIndexDefinition();

			migration.doMigration(template);

			verify(indexOperations).ensureIndex(indexDefinition);
		}

	}

	@Nested
	class TestBuildCompoundIndex {

		@Test
		void shouldSetInCreation() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexKeys()).containsEntry(M010_CreateIndexesInVorgang.FIELD_IN_CREATION, 1);
		}

		@Test
		void shouldSetOrganisationseinheitenId() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexKeys()).containsEntry(M010_CreateIndexesInVorgang.FIELD_ORGANISATIONSEINHEITEN_ID, 1);
		}

		@Test
		void shouldSetAssignedTo() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexKeys()).containsEntry(M010_CreateIndexesInVorgang.FIELD_ASSIGNED_TO, 1);
		}

		@Test
		void shouldSetStatus() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexKeys()).containsEntry(M010_CreateIndexesInVorgang.FIELD_STATUS, 1);
		}

		@Test
		void shouldSetCreatedAt() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexKeys()).containsEntry(M010_CreateIndexesInVorgang.FIELD_CREATED_AT, 1);
		}

		@Test
		void shouldSetName() {
			var indexDefinition = migration.buildCompoundIndexDefinition();

			assertThat(indexDefinition.getIndexOptions()).containsEntry("name", M010_CreateIndexesInVorgang.COMPOUND_INDEX_NAME);
		}
	}

}