/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.vorgang.callcontext.UserTestFactory;
import de.ozgcloud.vorgang.vorgang.AntragstellerTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgang.ZustaendigeStelleTestFactory;

class IndexedVorgangMapperTest {
	private IndexedVorgangMapper mapper = Mappers.getMapper(IndexedVorgangMapper.class);

	@Nested
	class TestToIndexedVorgang {

		@Test
		void shouldMapId() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getVorgangId()).isEqualTo(VorgangTestFactory.ID);
		}

		@Test
		void shouldMapAntragstellerName() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getAntragstellerName()).isEqualTo(AntragstellerTestFactory.NACHNAME);
		}

		@Test
		void shouldMapAntragstellerVorname() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getAntragstellerVorname()).isEqualTo(AntragstellerTestFactory.VORNAME);
		}

		@Test
		void shouldMaOrganistationseinheitenId() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getOrganisationseinheitenId()).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID);
		}

		@Test
		void shouldMapName() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getVorgangName()).isEqualTo(VorgangTestFactory.NAME);
		}

		@Test
		void shouldMapNummer() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getVorgangNummer()).isEqualTo(VorgangTestFactory.VORGANG_NUMMER);
		}

		@Test
		void shouldMapCreatedAt() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getCreatedAt()).isEqualTo(VorgangTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapAktenzeichen() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getAktenzeichen()).isEqualTo(VorgangTestFactory.AKTENZEICHEN);
		}

		@Test
		void shouldMapStatus() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getStatus()).isEqualTo(VorgangTestFactory.STATUS.name());
		}

		@Test
		void shouldMapAssignedTo() {
			IndexedVorgang indexedVorgang = mapper.fromVorgang(VorgangTestFactory.create());

			assertThat(indexedVorgang.getAssignedTo()).isEqualTo(UserTestFactory.ID);
		}
	}

	@Nested
	class TestFromIndexedVorgang {
		@Test
		void shouldMapId() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getId()).isEqualTo(VorgangTestFactory.ID);
		}

		@Test
		void shouldMapName() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getName()).isEqualTo(VorgangTestFactory.NAME);
		}

		@Test
		void shouldMapNummer() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getNummer()).isEqualTo(VorgangTestFactory.VORGANG_NUMMER);
		}

		@Test
		void shouldMapCreatedAt() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getCreatedAt()).isEqualTo(VorgangTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapAktenzeichen() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getAktenzeichen()).isEqualTo(VorgangTestFactory.AKTENZEICHEN);
		}

		@Test
		void shouldMapStatus() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getStatus()).isEqualTo(VorgangTestFactory.STATUS);
		}

		@Test
		void shouldMapAssignedTo() {
			var vorgang = mapper.toVorgangHeader(IndexedVorgangTestFactory.create());

			assertThat(vorgang.getAssignedTo()).isEqualTo(IndexedVorgangTestFactory.ASSIGNED_TO);
		}
	}

}
