/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import java.time.ZonedDateTime;

import de.ozgcloud.vorgang.callcontext.UserTestFactory;
import de.ozgcloud.vorgang.vorgang.AntragstellerTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgang.ZustaendigeStelleTestFactory;

class IndexedVorgangTestFactory {
	static final String AKTENZEICHEN = VorgangTestFactory.AKTENZEICHEN;
	static final String ANTRAGSTELLER_NAME = AntragstellerTestFactory.NACHNAME;
	static final String ANTRAGSTELLER_VORNAME = AntragstellerTestFactory.VORNAME;
	static final String ASSIGNED_TO = UserTestFactory.ID;
	static final ZonedDateTime CREATED_AT = VorgangTestFactory.CREATED_AT;
	static final String NAME = VorgangTestFactory.NAME;
	static final String VORGANG_NUMMER = VorgangTestFactory.VORGANG_NUMMER;
	static final String ORGANISATIONEINHEITEN_ID = ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID;
	static final String ID = VorgangTestFactory.ID;
	static final String STATUS = VorgangTestFactory.STATUS.name();

	static IndexedVorgang create() {
		return createBuilder().build();
	}

	static IndexedVorgang.IndexedVorgangBuilder createBuilder() {
		return new IndexedVorgang.IndexedVorgangBuilder()
				.aktenzeichen(AKTENZEICHEN)
				.antragstellerName(ANTRAGSTELLER_NAME)
				.antragstellerVorname(ANTRAGSTELLER_VORNAME)
				.assignedTo(ASSIGNED_TO)
				.createdAt(CREATED_AT)
				.vorgangName(NAME)
				.vorgangNummer(VORGANG_NUMMER)
				.organisationseinheitenId(ORGANISATIONEINHEITEN_ID)
				.status(STATUS)
				.vorgangId(ID);
	}

	public static String createScriptRemoveFields(String field) {
		return "ctx._source.remove('%s')".formatted(field);
	}
}
