/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.common.search;

import static org.assertj.core.api.Assertions.*;
import static org.awaitility.Awaitility.*;

import java.time.Duration;
import java.util.Optional;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.index.IndexInfo;

import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.vorgang.vorgang.Vorgang;

@SpringBootTest(properties = {
		"ozgcloud.mongodbsearch.enabled=true"
})
@DataITCase
class MongodbFullTextSearchIndexInitializerITCase {

	@Autowired
	private MongoOperations mongoOperations;

	@Test
	void shouldEnsureIndex() {
		await().atMost(Duration.ofSeconds(60)).untilAsserted(() ->
				assertThat(mongoOperations.indexOps(Vorgang.COLLECTION_NAME).getIndexInfo()).isNotEmpty());

		assertThat(getTextIndex()).isPresent();
	}

	private Optional<IndexInfo> getTextIndex() {
		return mongoOperations.indexOps(Vorgang.COLLECTION_NAME).getIndexInfo().stream()
				.filter(index -> MongodbFullTextSearchIndexInitializer.TEXT_INDEX_NAME.equals(index.getName())).findFirst();
	}

}