/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.gridfs.GridFsTemplate;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class OzgCloudFileTestUtil {

	private final MongoOperations mongoOperations;
	private final GridFsTemplate gridFsTemplate;

	public OzgCloudFile saveOzgCloudFile() {
		return saveOzgCloudFile(OzgCloudFileTestFactory.CONTENT);
	}

	public OzgCloudFile saveOzgCloudFile(byte[] content) {
		var contentId = storeFileContent(content);
		return mongoOperations.save(OzgCloudFileTestFactory.createBuilder().id(null).version(0).contentId(contentId).build());
	}

	public String storeFileContent(byte[] content) {
		return gridFsTemplate.store(new ByteArrayInputStream(content), UploadedFilesReferenceTestFactory.getGridFsFileName()).toString();
	}

	public InputStream getFileContentByFileName() {
		return gridFsTemplate.getResource(UploadedFilesReferenceTestFactory.getGridFsFileName()).getContent();
	}

	public InputStream getFileContent(String contentId) {
		var gridFSFile = gridFsTemplate.findOne(Query.query(Criteria.where(OzgCloudFileRepository.GRID_FS_FIELD_ID).is(contentId)));
		return gridFsTemplate.getResource(gridFSFile).getContent();
	}

	public void  dropFileCollections() {
		mongoOperations.dropCollection(OzgCloudFile.class);
		mongoOperations.dropCollection("fs.files");
		mongoOperations.dropCollection("fs.chunks");
	}
}
