/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.registry;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.vorgang.common.userconfig.UserConfigService;
import io.grpc.StatusRuntimeException;

class RegistrySchedulerTest {
	private static final String ORGANISATION_EINHEITEN_ID = "123456";
	private static final List<String> ORGANISATION_EINHEITEN_IDS = List.of(ORGANISATION_EINHEITEN_ID);

	@Spy
	@InjectMocks
	private RegistryScheduler registryScheduler;

	@Mock
	private RegistryService registryService;

	@Mock
	private UserConfigService userConfigService;

	@DisplayName("Test the registering the VorgangManager")
	@Nested
	class TestScheduler {

		@Test
		void shouldCallService() {
			when(userConfigService.getSupportedOrganisationsEinheiten()).thenReturn(ORGANISATION_EINHEITEN_IDS);

			registryScheduler.register();

			verify(registryService).registerVorgangManager(anyList());
		}

		@Test
		void shouldHandleExceptionWhenLoadingOrganisationEinheiten() {
			when(userConfigService.getSupportedOrganisationsEinheiten()).thenThrow(StatusRuntimeException.class);

			registryScheduler.register();

			verify(registryService, never()).registerVorgangManager(anyList());
		}

		@Test
		void shouldCallGetOrganisationEinheitenIds() {
			when(userConfigService.getSupportedOrganisationsEinheiten()).thenReturn(ORGANISATION_EINHEITEN_IDS);

			registryScheduler.register();

			verify(userConfigService).getSupportedOrganisationsEinheiten();
		}
	}
}
