/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.registry;

import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;

import de.ozgcloud.vorgang.VorgangManagerProperties;

class RegistryServiceTest {

	private static final String ORGANISATION_EINHEITEN_ID = "123456";
	private static final List<String> ORGANISATION_EINHEITEN_IDS = List.of(ORGANISATION_EINHEITEN_ID);
	private static final String VORGANG_MANAGER_ADDRESS = "vorgang-manager-address";

	@Spy
	@InjectMocks
	private RegistryService registryService;

	@Mock
	private ZufiRemoteService zufiRemoteService;
	@Mock
	private VorgangManagerProperties vorgangManagerProperties;

	@Test
	void shouldCallZuFiRemoteService() {
		when(vorgangManagerProperties.getAddress()).thenReturn(VORGANG_MANAGER_ADDRESS);

		registryService.registerVorgangManager(ORGANISATION_EINHEITEN_IDS);

		verify(zufiRemoteService).registerVorgangManager(ORGANISATION_EINHEITEN_IDS, VORGANG_MANAGER_ADDRESS);
	}

	@DisplayName("should not call zufi remote service when organisationseinheitenIds is empty")
	@Test
	void shouldNotCallRegisterAddress() {
		registryService.registerVorgangManager(Collections.emptyList());

		Mockito.verifyNoInteractions(zufiRemoteService);
	}

}
