/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class FindVorgangRequestMapperTest {

	@InjectMocks
	private final FindVorgangRequestMapper mapper = Mappers.getMapper(FindVorgangRequestMapper.class);
	@Mock
	private FilterByMapper filterByMapper;
	@Mock
	private FindVorgangQueryMapper findVorgangQueryMapper;

	@DisplayName("Map from find vorgang request")
	@Nested
	class TestMapFromFindVorgangRequest {

		@BeforeEach
		void mockFilterByMapper() {
			when(filterByMapper.fromFilterBy(any())).thenReturn(FilterCriteriaTestFactory.create());
		}

		@Test
		void shouldMap() {
			var result = mapper.fromFindVorgangRequest(GrpcFindVorgangRequestTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(FindVorgangRequestTestFactory.create());
		}

		@Test
		void shouldNullEmptySearchBy() {
			var result = mapper.fromFindVorgangRequest(GrpcFindVorgangRequestTestFactory.createBuilder().setSearchBy(StringUtils.EMPTY).build());

			assertThat(result).usingRecursiveComparison().isEqualTo(FindVorgangRequestTestFactory.createBuilder().searchBy(null).build());
		}

		@Test
		void shouldCallFilterMapper() {
			mapper.fromFindVorgangRequest(GrpcFindVorgangRequestTestFactory.create());

			verify(filterByMapper).fromFilterBy(GrpcFilterByTestFactory.create());
		}

		@Test
		void shouldMapQuery() {
			var expectedQuery = GrpcQueryTestFactory.create();
			var findVorgangRequest = GrpcFindVorgangRequestTestFactory.createBuilder().setQuery(expectedQuery).build();

			mapper.fromFindVorgangRequest(findVorgangRequest);

			verify(findVorgangQueryMapper).fromQuery(expectedQuery);
		}
	}
}
