/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static de.ozgcloud.vorgang.vorgang.VorgangService.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.test.context.bean.override.mockito.MockitoSpyBean;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.notification.antragsteller.AntragstellerNotificationEventListener;
import de.ozgcloud.notification.user.UserNotificationEventListener;
import de.ozgcloud.processor.ProcessorManagerConfiguration;
import de.ozgcloud.processor.processor.ProcessorService;
import de.ozgcloud.processor.vorgang.ProcessorVorgangMapper;
import de.ozgcloud.vorgang.command.CommandCreatedEventTestFactory;
import de.ozgcloud.vorgang.command.CommandTestFactory;
import de.ozgcloud.vorgang.command.Order;
import de.ozgcloud.vorgang.files.FileService;

@ITCase
@SpringBootTest(properties = {
		"ozgcloud.processors[0].address=http://test1",
		"ozgcloud.processors[0].name=processor1",
		"ozgcloud.processors[0].forms[0].formEngineName=formEngineTest1",
		"ozgcloud.processors[0].forms[0].formId=ID1"
})
class VorgangEventListenerITCase {

	@Autowired
	private ApplicationEventPublisher publisher;

	@MockitoSpyBean
	private VorgangEventListener listener;
	@MockitoSpyBean
	private AntragstellerNotificationEventListener antragstellerNotificationEventListener;
	@MockitoSpyBean
	private UserNotificationEventListener userNotificationEventListener;

	@MockitoSpyBean
	private VorgangService service;
	@MockitoSpyBean
	private VorgangHeaderService vorgangHeaderService;
	@MockitoSpyBean
	private FileService fileService;

	@MockitoSpyBean
	private ProcessorService vorgangProcessorService;
	@MockitoSpyBean
	@Qualifier(ProcessorManagerConfiguration.OZGCLOUD_VORGANG_SERVICE_NAME)
	private OzgCloudVorgangService ozgCloudVorgangService;
	@MockitoSpyBean
	private ProcessorVorgangMapper processorVorgangMapper;

	@Nested
	class TestOnVorgangLoeschen {

		@BeforeEach
		void init() {
			doNothing().when(listener).publishEvent(any());
		}

		@Test
		void shouldCallService() {
			publisher.publishEvent(CommandCreatedEventTestFactory.create(CommandTestFactory.createBuilder().order("VORGANG_LOESCHEN").build()));

			verify(service, timeout(500)).deleteVorgang(VorgangTestFactory.ID);
		}

	}

	@Nested
	class TestOnSetAktenzeichen {

		private static final Command SET_AKTENZEICHEN_COMMAND = CommandTestFactory.createBuilder().order(Order.SET_AKTENZEICHEN.name())
				.bodyObject(Map.of(BODY_OBJECT_AKTENZEICHEN, VorgangTestFactory.AKTENZEICHEN)).build();
		private static final CommandCreatedEvent SET_AKTENZEICHEN_EVENT = CommandCreatedEventTestFactory.create(SET_AKTENZEICHEN_COMMAND);

		@BeforeEach
		void init() {
			doNothing().when(listener).onSetAktenzeichen(any());
		}

		@Test
		void shouldCallOnSetAktenzeichen() {
			publisher.publishEvent(SET_AKTENZEICHEN_EVENT);

			verify(listener, timeout(500)).onSetAktenzeichen(SET_AKTENZEICHEN_EVENT);
		}

		@Test
		void shouldNotCallOnSetAktenzeichen() {
			publisher.publishEvent(CommandCreatedEventTestFactory.create(CommandTestFactory.createBuilder().order("NOT_SET_AKTENZEICHEN").build()));

			verify(listener, never()).onSetAktenzeichen(any());
		}
	}
}
