/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.clientattribute.ClientAttributeMapper;
import de.ozgcloud.vorgang.servicekonto.GrpcServiceKontoTestFactory;
import de.ozgcloud.vorgang.servicekonto.ServiceKontoMapper;

class VorgangWithEingangMapperTest {

	@InjectMocks
	private VorgangWithEingangMapper mapper = Mappers.getMapper(VorgangWithEingangMapper.class);
	@Mock
	private EingangMapper eingangMapper;
	@Mock
	private ClientAttributeMapper clientAttributeMapper;
	@Mock
	private ServiceKontoMapper serviceKontoMapper;

	@Nested
	class TestToVorgangWithEingang {

		@BeforeEach
		void mockEingangMapper() {
			when(eingangMapper.toGrpc(any(Eingang.class))).thenReturn(GrpcEingangTestFactory.create());
			when(serviceKontoMapper.toServiceKonto(any())).thenReturn(GrpcServiceKontoTestFactory.create());
		}

		@Test
		void shouldCallEingangMapper() {
			callMapper();

			verify(eingangMapper).toGrpc(VorgangTestFactory.EINGANG);
		}

		@Test
		void shouldMapVorgangValues() {
			var vorgangWithEingang = callMapper();

			assertThat(vorgangWithEingang).usingRecursiveComparison()
					.ignoringFields("eingang_", "clientAttributes_", "memoizedHashCode")
					.isEqualTo(GrpcVorgangWithEingangTestFactory.create());
		}

		@Test
		void shouldMapEingangValue() {
			var eingang = callMapper().getEingang();

			assertThat(eingang).usingRecursiveComparison().isEqualTo(GrpcEingangTestFactory.create());
		}

		private GrpcVorgangWithEingang callMapper() {
			var vorgang = VorgangTestFactory.createBuilder().header(VorgangHeadTestFactory.createBuilder().lock(VorgangHeadTestFactory.LOCK).build());
			return mapper.toVorgangWithEingang(vorgang.build());
		}
	}
}