/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import de.ozgcloud.eingang.forwarding.GrpcAttachment;
import de.ozgcloud.eingang.forwarding.GrpcRepresentation;
import de.ozgcloud.eingang.forwarding.GrpcRouteForwardingRequest;

class GrpcRouteForwardingRequestTestFactory {

	public static final GrpcAttachment ATTACHMENT_METADATA = GrpcAttachmentTestFactory.createWithFile();
	public static final GrpcAttachment ATTACHMENT_CONTENT = GrpcAttachmentTestFactory.createWithContent();
	public static final GrpcRepresentation REPRESENTATION_METADATA = GrpcRepresentationTestFactory.createWithFile();
	public static final GrpcRepresentation REPRESENTATION_CONTENT = GrpcRepresentationTestFactory.createWithContent();

	public static GrpcRouteForwardingRequest create() {
		return createBuilder().build();
	}

	public static GrpcRouteForwardingRequest.Builder createBuilder() {
		return GrpcRouteForwardingRequest.newBuilder()
				.setRouteForwarding(GrpcRouteForwardingTestFactory.create());
	}

	public static GrpcRouteForwardingRequest createWithAttachmentMetadata() {
		return GrpcRouteForwardingRequest.newBuilder()
				.setAttachment(ATTACHMENT_METADATA)
				.build();
	}

	public static GrpcRouteForwardingRequest createWithAttachmentContent() {
		return GrpcRouteForwardingRequest.newBuilder()
				.setAttachment(ATTACHMENT_CONTENT)
				.build();
	}

	public static GrpcRouteForwardingRequest createWithRepresentationMetadata() {
		return GrpcRouteForwardingRequest.newBuilder()
				.setRepresentation(REPRESENTATION_METADATA)
				.build();
	}

	public static GrpcRouteForwardingRequest createWithRepresentationContent() {
		return GrpcRouteForwardingRequest.newBuilder()
				.setRepresentation(REPRESENTATION_CONTENT)
				.build();
	}
}
